#ifndef DISPLAYDRIVER_H
#define DISPLAYDRIVER_H

#include <X11/Xlib.h>

class KeyHandler;

/**
 * Abstract base class for display drivers.  Drivers are the classes that
 * manage the outer context stuff (for example, there is a WurldGtkDriver for
 * running in a Gtk window and a WurldSDLDriver for running on top of SDL).
 */
class DisplayDriver {
   protected:
      KeyHandler *keyHandler;

   public:
      struct Size {
         int width;
         int height;

         Size(int width, int height) : width(width), height(height) {}
      };

      DisplayDriver() : keyHandler(0) {}

      virtual ~DisplayDriver() {}

      /** 
       * Sets the display size.  Must be called before main();
       */
      virtual void setDisplaySize(int width, int height) = 0;

      /** Returns the display size */
      virtual Size getSize() const = 0;

      /** hide the mouse cursor */
      virtual void hideMouse() = 0;

      /** warps the mouse to the center of the display */
      virtual void warpMouse(int x, int y) = 0;

      /** grabs (or ungrabs) the mouse pointer */
      virtual void grabMouse(bool grab = true) = 0;

      /** Sets the key handler associated with the driver */
      void setKeyHandler(KeyHandler *newKeyHandler) {
         keyHandler = newKeyHandler;
      }

      /** Returns the currently active key handler. */
      KeyHandler *getKeyHandler() const {
         return keyHandler;
      }

      virtual int main(int argc, const char **argv) = 0;
};

#endif
