
#include "Graphics.h"
#include "Font.h"
#include <GL/gl.h>

Graphics Graphics::instance;

Graphics::Graphics() : fixed(0) {}

Graphics::~Graphics() {
   if (fixed)
      delete fixed;
}

void Graphics::initialize(int initWidth, int initHeight) {
   width = initWidth;
   height = initHeight;
   glViewport(0, 0, width, height);

   glClearColor(0, 0, 0, 0);
   glClearDepth(1.0);
   glShadeModel(GL_SMOOTH);
   glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
   glEnable(GL_DEPTH_TEST);
   glEnable(GL_TEXTURE_2D);
   glDepthFunc(GL_LEQUAL);
   glEnable(GL_LIGHTING);
   glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
}

void Graphics::enableLighting(bool enabled) {
   if (enabled)
      glEnable(GL_LIGHTING);
   else
      glDisable(GL_LIGHTING);
   isLightingEnabled = enabled;
}

void Graphics::drawText(const char *text, int length) {
   // use the string length if so indicated
   if (length == -1)
      length = strlen(text);

   glPushAttrib(GL_LIST_BIT);
   getFont();
   fixed->setBase();
   glCallLists(length, GL_UNSIGNED_BYTE, text);
   glPopAttrib();
}

void Graphics::moveTo(Float x, Float y) {
   glRasterPos2f(x, y);
}

void Graphics::setColor(const Color &color) {
   glColor<Float>(color.red, color.green, color.blue);
}

BitmapFont &Graphics::getFont() const {
   if (!fixed)
      fixed = new BitmapFont();
   return *fixed;
}
