
#ifndef GRAPHICS_H
#define GRAPHICS_H

#include "basictypes.h"
#include "Color.h"

class BitmapFont;

class Graphics {
   private:
      static Graphics instance;

      // the default font
      mutable BitmapFont *fixed;

      bool isLightingEnabled;

      // screen size
      unsigned width, height;

   public:

      Graphics();

      ~Graphics();

      /** Returns the graphics singleton. */
      static Graphics &getInstance() {
	 return instance;
      }

      /**
       * Perform all GL one-time initialization functions.
       */
      void initialize(int width, int height);

      /**
       * Enable (or disable) lighting.
       */
      void enableLighting(bool enabled = true);

      /**
       * Returns true if the lighting is enabled, false if not.
       */
      bool lightingEnabled() {
         return isLightingEnabled;
      }

      /**
       * Draw the given text at the current position.
       * @param text text to draw
       * @param length the number of characters to draw.  If a value of -1
       *               is specified, use the string length of text.
       */
      void drawText(const char *text, int length = -1);

      /** Move to a 2-d position on the raster display. */
      void moveTo(Float x, Float y);

      /** Sets the current drawing color */
      void setColor(const Color &color);

      /** 
       * Convert from pixels to a floating point screen distances.
       * Note that these methods convert to distances, not coordinates.  To
       * create a coordinate, subtract 1 from the result.
       */
      /** @{ */
      Float pixelToXFloat(unsigned x) {
         return 2 * static_cast<float>(x) / width;
      }

      Float pixelToYFloat(unsigned y) {
         return 2 * static_cast<float>(y) / height;
      }
      /** @} */

      /**
       * Returns the current font.
       */
      BitmapFont &getFont() const;

      /**
       * Returns the maximum distance between two verteces on the same polygon
       * when lighting is enabled.  If the renderer discovers a polygon that
       * is larger than this, it will break it up into smaller polygons that
       * can be lit more effectively.
       */
      Float getMaxVertexDistance() const {
         // XXX wuss out on this for now...
         return 30;
      }
};

#endif
