/*===========================================================================*\

   $Id$

   Copyright (C) 2007 Michael A. Muller

   Permission is granted to use, modify and redistribute this code,
   providing that the following conditions are met:
 
   1) This copyright/licensing notice must remain intact.
   2) If the code is modified and redistributed, the modifications must 
   include documentation indicating that the code has been modified.
   3) The author(s) of this code must be indemnified and held harmless
   against any damage resulting from the use of this code.

   This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
   warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

\*===========================================================================*/

#include "PythonKeyCommand.h"

using namespace wurld;

PythonKeyCommand::PythonKeyCommand(PyObject *onKeyPress, 
                                   PyObject *onKeyRelease
                                   ) : 
   onKeyPress(onKeyPress),
   onKeyRelease(onKeyRelease) {
   if (onKeyPress)
      Py_INCREF(onKeyPress);
   if (onKeyRelease)
      Py_INCREF(onKeyRelease);
}

PythonKeyCommand::~PythonKeyCommand() {
   Py_XDECREF(onKeyPress);
   Py_XDECREF(onKeyRelease);
}

void PythonKeyCommand::handleKeyPress(KeyCode keyCode) {
   if (onKeyPress && onKeyPress != Py_None) {

      // build the argument list
      PyObject *args = Py_BuildValue("(i)", keyCode);
      if (!args) {
         PyErr_Print();
         return;
      }

      PyObject *result = PyObject_CallObject(onKeyPress, args);
      Py_DECREF(args);

      if (!result)
         PyErr_Print();
      else
         Py_DECREF(result);
   }
}

void PythonKeyCommand::handleKeyRelease(KeyCode keyCode) {
   if (onKeyRelease && onKeyRelease != Py_None) {

      // build the argument list
      PyObject *args = Py_BuildValue("(i)", keyCode);
      if (!args) {
         PyErr_Print();
         return;
      }

      PyObject *result = PyObject_CallObject(onKeyRelease, args);
      Py_DECREF(args);

      if (!result)
         PyErr_Print();
      else
         Py_DECREF(result);
   }
}
