#ifndef PYTHONPLUGININTERNAL_H
#define PYTHONPLUGININTERNAL_H

#include <spug/RCPtr.h>
#include "Vector.h"

typedef struct _object PyObject;

SPUG_RCPTR(KeyCommand);
KeyCommandPtr createKeyCommand(PyObject *onKeyPress, PyObject *onKeyRelease);

class DisplayDriver;
DisplayDriver *getDisplayDriver();

class GameEngine;
GameEngine *getGameEngine();

class RotMatrix;
RotMatrix createYawRotMatrix(Float degrees);
RotMatrix createPitchRotMatrix(Float degrees);
RotMatrix createRollRotMatrix(Float degrees);

Vector createVector(Float x, Float y, Float z);

SPUG_RCPTR(Texture);
TexturePtr createTexture(const char *filename);

SPUG_RCPTR(Material);
MaterialPtr createTextureMaterial(const TexturePtr &texture, 
				  Float xppus, 
				  Float yppus
				  );

SPUG_RCPTR(PolyObj);
PolyObjPtr createQuad(Vector c0, Vector c1, Vector c2, Vector c3, 
		      const MaterialPtr &material
		      );

SPUG_RCPTR(Model);
ModelPtr createModel();

SPUG_RCPTR(Moveable);
MoveablePtr createMoveable();

SPUG_RCPTR(RenderableObject);
RenderableObjectPtr createSimpleText(const char *text, Float left, Float top,
                                     Float width,
                                     Float height
                                     );

RenderableObjectPtr createFPSDisplay(GameEngine *engine);

class PythonObject;
PythonObject *createPythonObject();

class Graphics;
Graphics *getGraphics();

#endif
