
#include "QuadObj.h"

#include <iostream>
#include "PNG.h"
#include "Coord.h"
#include "PlanarMesh.h"
#include "Graphics.h"

using namespace std;

spug::Tracer::Level QuadObj::tesselation = 
   spug::Tracer::get().getLevel("QuadObj.tesselation");

void QuadObj::makeMap() {
   // XXX this is lame - need to learn how to map from 3d coords to the plane
   // that they occupy
   map = new Coord[4];
   Float sdist = (coord1 - coord0).size();
   Float tdist = (coord3 - coord0).size();
   map[1].x = sdist;
   map[2] = Coord(sdist, tdist);
   map[3].y = tdist;
}

QuadObj::QuadObj(const QuadObj &other) : Quad(other), PolyObj(other), map(0) {
   if (other.map) {
      map = new Coord[4];
      for (int i = 0; i < 4; ++i)
         map[i] = other.map[i];
   }
}

QuadObj::~QuadObj() {
   if (map) delete map;
}

void QuadObj::makeMesh(const PlanarMesh::Quad &quad) {
   typedef PlanarMesh::Vertex V;
   typedef std::pair<V,V> Side;

   // store the sides as an array of vertex pairs
   Side sides[4];
   for (int i = 0; i < 4; ++i)
      sides[i] = Side(quad[i], quad[(i + 1) % 4]);

   // calculate the difference vectors
   V diffs[4];
   for (int i = 0; i < 4; ++i)
      diffs[i] = sides[i].second - sides[i].first;

   // calculate the lengths of the sides
   Float lengths[4];
   for (int i = 0; i < 4; ++i)
      lengths[i] = diffs[i].size();

   // find the longest side
   int longest = 0;
   for (int i = 1; i < 4; ++i)
      if (lengths[i] > lengths[longest])
         longest = i;

   SPUG_TRACE(tesselation, 
              " quad is " << quad[0] << quad[1] << quad[2] << quad[3]
              );

   // if the longest side is greater than the max
   // subdivide into two quads
   if (lengths[longest] > Graphics::getInstance().getMaxVertexDistance()) {
      int opposite = (longest + 2) % 4;
      unsigned d0 = 
         mesh->addVertex(sides[longest].first + diffs[longest] * 0.5);
      unsigned d1 = 
         mesh->addVertex(sides[opposite].second + diffs[opposite] * -0.5);

      PlanarMesh::Quad child0(quad.coords[longest], d0, d1, 
                              quad.coords[(longest + 3) % 4]
                              );
      mesh->addQuad(child0);
      SPUG_TRACE(tesselation, 
                 " child0 is " << child0[0] << child0[1] << child0[2] << 
                  child0[3]
                 );
      makeMesh(child0);

      PlanarMesh::Quad child1(d0, quad.coords[(longest + 1) % 4], 
                              quad.coords[opposite], d1
                              );
      mesh->addQuad(child1);
      SPUG_TRACE(tesselation, 
                 "child1 is " << child1[0] << child1[1] << child1[2] << 
                  child1[3]
                 );
      makeMesh(child1);

      // remove this quad
      mesh->removeQuad(quad);
   } else {
      SPUG_TRACE(tesselation, "Hit bottom");
   }
}

#if 0
MeshPtr QuadObj::makeMesh(const Quad &quad) {

   typedef Mesh::Vertex V;
   typedef std::pair<V, V> Side;

   // store the sides as an array of vertex pairs
   Side sides[4] = {
      Side(V(quad.coord0, map[0]), V(quad.coord1, map[1])),
      Side(V(quad.coord1, map[1]), V(quad.coord2, map[2])),
      Side(V(quad.coord2, map[2]), V(quad.coord3, map[3])),
      Side(V(quad.coord3, map[3]), V(quad.coord0, map[0]))
   };

   // calculate the difference vectors
   V diffs[4];
   for (int i = 0; i < 4; ++i)
      diffs[i] = sides[i].second - sides[i].first;

   // calculate the lengths of the sides
   Float lengths[4];
   for (int i = 0; i < 4; ++i)
      lengths[i] = diffs[i].size();

   // find the longest side
   int longest = 0;
   for (int i = 1; i < 4; ++i)
      if (lengths[i] > lengths[longest])
         longest = i;

   // find the next longest adjacent side XXX not sure we need this
   int after = (longest + 1) % 4, before = (longest - 1) % 4;
   int nextLongest = lengths[before] > lengths[after] ? before : after;
   int opposite = (longest + 2) % 4;

   Float maxLen = Graphics::getInstance().getMaxVertexDistance();
   assert(lengths[longest] > maxLen);

   // long side is "S", longest adjacent is "T"

   // slice the quad into strips along the longest edge
   vector<Side> sides;
   sides.push_back(sides[before]);
   width0 = length[longest];
   width1 = lenght[opposite];
   V sPos0 = sides[longest].first, sPos1 = sides[opposite].second;
   V svec0 = diffs[longest] * (maxLen / width0);
   V svec1 = diffs[opposite] * -(maxLen / width1);
   if (width > maxLen) {
      sPos0 += svec0;
      width -= maxLen;
   } else {
      sPos0 = sides[longest].second;
   }

   if (width1 >= maxLen) {
      sPos1 += svec1;
   } else {
      // the new line segment forms a triangle with the rest of the quad
      sPos0 + diffs[before];
      sPos1 += svec
   }
   width1 -= maxLen;

}
#endif

void QuadObj::render(Scene &scene) {

   // if we don't have a map, generate one
   if (!map) makeMap();

   // set the surface material
   material->makeCurrent();

   // if lighting is enabled and this quad is bigger than the prescribed size,
   // we need to render the quad as a mesh

   Graphics &grap = Graphics::getInstance();
   if (grap.lightingEnabled()) {

      // if we haven't checked the size, do so now
      if (!checkedSize) {
         Float maxSize = grap.getMaxVertexDistance();
         if ((coord1 - coord0).size() > maxSize ||
             (coord2 - coord1).size() > maxSize ||
             (coord3 - coord2).size() > maxSize ||
             (coord0 - coord3).size() > maxSize
             ) {

            // too big.  Create the mesh.
            SPUG_TRACE(tesselation,
                       "====================== Creating mesh ==============="
                        "=========="
                       );
            mesh = new PlanarMesh(normal(), material);
            PlanarMesh::Quad quad(
               mesh->addVertex(PlanarMesh::Vertex(coord0, map[0])),
               mesh->addVertex(PlanarMesh::Vertex(coord1, map[1])),
               mesh->addVertex(PlanarMesh::Vertex(coord2, map[2])),
               mesh->addVertex(PlanarMesh::Vertex(coord3, map[3]))
            );
            mesh->addQuad(quad);
            makeMesh(quad);
         } else {
            SPUG_TRACE(tesselation, 
                       "omitting mesh generation for " <<
                        coord0 << coord1 << coord2 << coord3
                       );
         }

         checkedSize = true;
      }

      // if we ended up with a mesh, render the mesh and quit
      if (mesh) {
         mesh->render(scene);
         return;
      }
   }

   // do the normal cube rendering
   glBegin(GL_QUADS);
      Vector norm = normal();
      glNormal<Float>(norm.x, norm.y, norm.z);
      material->setCoord(map[0]);
      glVertex(coord0.x, coord0.y, coord0.z);
      material->setCoord(map[1]);
      glVertex(coord1.x, coord1.y, coord1.z);
      material->setCoord(map[2]);
      glVertex(coord2.x, coord2.y, coord2.z);
      material->setCoord(map[3]);
      glVertex(coord3.x, coord3.y, coord3.z);
   glEnd();
}

bool QuadObj::intersects(const Vector &origin, const Vector &direction,
			 Vector &intersection,
			 Float &distance
			 ) {
   // XXX refactor Tri::intersects so we can make this more efficient
   return 
      Tri(coord0, coord1, coord2).intersects(origin, direction, intersection, distance) ||
      Tri(coord0, coord2, coord3).intersects(origin, direction, intersection, distance);
}

AxisBox QuadObj::getBoundingBox() const {
   AxisBox box(false);
   box.expandTo(coord0);
   box.expandTo(coord1);
   box.expandTo(coord2);
   box.expandTo(coord3);

   return box;
}
