/*===========================================================================*\

   $Id$

   Copyright (C) 2007 Michael A. Muller

   Permission is granted to use, modify and redistribute this code,
   providing that the following conditions are met:
 
   1) This copyright/licensing notice must remain intact.
   2) If the code is modified and redistributed, the modifications must 
   include documentation indicating that the code has been modified.
   3) The author(s) of this code must be indemnified and held harmless
   against any damage resulting from the use of this code.

   This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
   warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

\*===========================================================================*/

#ifndef TEXTMODEL_H
#define TEXTMODEL_H

#include <spug/RCBase.h>
#include <spug/RCPtr.h>
#include "Graphics.h"
#include "Font.h"

SPUG_RCPTR(TextModel);

/**
 * Basic interface for the model object for the TextPane.
 */
class TextModel : public spug::RCBase {
   public:
      /** Text cursor class. */
      class Cursor {
         friend class TextModel;
         private:
            class BoolWrapper {
               public:
                  void trueFunc() {}
            };
            typedef void (BoolWrapper::* Bool)();

            const char *pos, *end;

            Cursor(const char *pos, const char *end) : pos(pos), end(end) {}

         public:
            Cursor() : pos(0), end(0) {}

            char operator *() const {
               return *pos;
            }

            Cursor operator +(int val) const {
               const char *tmp = pos + val;
               if (tmp > end)
                  return Cursor(0, end);
               else
                  return Cursor(tmp, end);
            }

            int operator -(const Cursor &other) const {
               if (pos)
                  return pos - other.pos;
               else if (other.pos)
                  return other.end - other.pos;
               else
                  return 0;
            }

            operator Bool() const {
               return pos ? &BoolWrapper::trueFunc : 0;
            }

            Cursor &operator ++() {
               ++pos;
               if (pos >= end)
                  pos = 0;
               return *this;
            }

            const char *c_str() const {
               return pos;
            }
      };

      /** Change listener interface */
      class ChangeListener {
         public:
            void onChange(TextModel source, Cursor start, Cursor end);
      };

      virtual ~TextModel() {}

      void setChangeListener(ChangeListener *listener) {
         changeListener = listener;
      }

      /**
       * Returns a cursor pointing to the specified row and column.
       */
      virtual Cursor getPos(unsigned row, unsigned column) const = 0;

   protected:
      /**
       * Provides access to the Cursor destructor to the derived classes.
       */
      static Cursor createCursor(const char *pos, const char *end) {
         return Cursor(pos, end);
      }

   private:
      ChangeListener *changeListener;
};

#endif
