/*===========================================================================*\

   $Id$

   Copyright (C) 2007 Michael A. Muller

   Permission is granted to use, modify and redistribute this code,
   providing that the following conditions are met:
 
   1) This copyright/licensing notice must remain intact.
   2) If the code is modified and redistributed, the modifications must 
   include documentation indicating that the code has been modified.
   3) The author(s) of this code must be indemnified and held harmless
   against any damage resulting from the use of this code.

   This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
   warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

\*===========================================================================*/

#include "TextPane.h"

TextPane::TextPane(const TextModelPtr &model, Float left, Float top, 
                   Float width, 
                   Float height) :
   model(model),
   row(0),
   column(0),
   top(top),
   left(left),
   width(width),
   height(height) {
}

void TextPane::render(Scene &scene) {
   TextModel::Cursor cur = model->getPos(row, column);

   Graphics &grap = Graphics::getInstance();
   BitmapFont &font = grap.getFont();
   Float baseline = top - grap.pixelToYFloat(font.getMaxAscender());
   Float maxHeight = 
      grap.pixelToYFloat(font.getMaxAscender() + font.getMaxDescender());

   // fill the pane until we run out of text or vertical space
   while (cur && baseline < top + height) {
      TextModel::Cursor start = cur, stop;

      // find the next stop position (end of buffer or end of line)
      for (; cur && *cur != '\n'; ++cur) ;
      stop = cur;

      // draw the line of text
      grap.moveTo(left, baseline);
      grap.drawText(start.c_str(), stop - start);

      // next baseline
      baseline = 
         baseline + 
         grap.pixelToYFloat(font.getMaxAscender() + font.getMaxDescender());

      if (cur) cur = stop + 1;
   }
}
