#==============================================================================
#
#  $Id$
#
"""
   Some basic utilities.
"""
#
#  Copyright (C) 2007 Michael A. Muller
#
#  Permission is granted to use, modify and redistribute this code,
#  providing that the following conditions are met:
#
#  1) This copyright/licensing notice must remain intact.
#  2) If the code is modified and redistributed, the modifications must 
#  include documentation indicating that the code has been modified.
#  3) The author(s) of this code must be indemnified and held harmless
#  against any damage resulting from the use of this code.
#
#  This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
#  warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#
#==============================================================================

import wurld

def vecToStr(vec):
   return '(%g, %g, %g)' % (vec.x, vec.y, vec.z)

class Sprite(object):

   def __init__(self, model = None, velocity = None):
      self.__moveable = wurld.createMoveable()
      self.__pythonObject = wurld.createPythonObject()
      self.__pythonObject.setMoveable(self.__moveable)

      self.__model = None
      if model:
         self.model = model

      self.__gotVelocity = False
      if velocity:
         self.velocity = velocity

   def __getModel(self):
      return self.__model

   def __setModel(self, model):
      if not self.__model:
         wurld.getGameEngine().addRenderableObject(self.__pythonObject)
      self.__model = model
      self.__moveable.setModel(model)

   model = property(__getModel, __setModel, None, "the associated model")

   def __getOrigin(self):
      return self.__moveable.getPosition()

   def __setOrigin(self, pos):
      self.__moveable.setPosition(pos)

   origin = property(__getOrigin, __setOrigin, None, 
                     "the position of the model")

   def __getVelocity(self):
      return self.__moveable.getVelocity()

   def __setVelocity(self, velocity):
      if not self.__gotVelocity:
         # once velocity gets set, we need to become an engine object so that
         # motion starts happening
         self.__gotVelocity = True
         wurld.getGameEngine().addGameObject(self.__pythonObject)
      self.__moveable.setVelocity(velocity)

   velocity = property(__getVelocity, __setVelocity, None, 
                       "the velocity of the object"
                       )

   def __getRotation(self):
      return self.__moveable.getRotation()

   def __setRotation(self, rot):
      self.__moveable.setRotation(rot)

   rotation = property(__getRotation, __setRotation, None,
                       "the object's rotation"
                       )

   def __setEngineFrameCallback(self, callback):
      self.__pythonObject.setEngineFrameCallback(callback)

   engineFrameCallback = property(None, __setEngineFrameCallback, None,
                                  "Function to be called every engine frame"
                                  )

def showModel(model):
   """
      Wraps the model in a moveable and a python object and adds it to the
      renderables.
   """
   moveable = wurld.createMoveable()
   moveable.setModel(model)
   obj = wurld.createPythonObject()
   obj.setMoveable(moveable)
   wurld.getGameEngine().addRenderableObject(obj)
   return obj

class Rot:
   """
      A class to make dealing with rotation matrices a little less verbose.
   """

   def __init__(self, rotMatrix):
      self.mtx = rotMatrix

   @staticmethod
   def yaw(deg):
      return Rot(wurld.createYawRotMatrix(deg))

   @staticmethod
   def pitch(deg):
      return Rot(wurld.createPitchRotMatrix(deg))

   @staticmethod
   def roll(deg):
      return Rot(wurld.createRollRotMatrix(deg))

   def __mul__(self, other):
      temp = Rot.yaw(0)
      temp.mtx.makeProduct(self.mtx, other.mtx)
      return temp

   def rotate(self, vector):
      temp = wurld.createVector(0, 0, 0)
      self.mtx.rotate(vector, temp)
      return temp

class TextPane:

   def __init__(self, x, y, width, height):
      self.x = x
      self.y = y
      self.width = width
      self.height = height
      self.__obj = wurld.createSimpleText("", x, y, width, height)
      wurld.getGameEngine().addHUDSticker(self.__obj)
      self.__buffer = ''

   def write(self, text):
      self.__buffer += text

      # replace the existing simple text object with a new one
      engine = wurld.getGameEngine()
      engine.removeHUDSticker(self.__obj)
      self.__obj = \
         wurld.createSimpleText(self.__buffer, self.x, self.y,
                                self.width, 
                                self.height
                                )
      engine.addHUDSticker(self.__obj)
